<?php

/* WLUG Wiki customisation.
   This blocks requests based on customised variables below.
   We disallow requests at two times:
   1) disallow any access to the wiki pages (normally for obvious abuses
      such as gratuitous crawling/spam bots); and
   2) When someone wants to save an edited page (normally for persistent
      wiki spammers). We do the check on save instead of merely on edit
      so that they can waste some of their own time.

   Variables (arrays):
   * $access_blacklist_ips
   * $access_blacklist_agents
   * $edit_blacklist_ip_rxs

   Functions:
   * check_access_blacklist() # doesn't return if access denied
   * check_edit_spam( $old_text, $new_text ) # doesn't return if spam (also
   	checks if the IP is blacklisted for edits)

   Further ideas if needed:
   * look up link targets in SORBS
   * spam-assassin/bayes on content?


   This file is used by phpwiki (in lib/display.php) and also by
   wlug's installfest registration.php script.

   -- John McPherson

 */



# if you add an IP here, give a date, and say why!
# if netstat(1) doesn't show you the whole IP, try looking in
# /proc/net/ip_conntrack
# remember, apache is using ipv6, so prepend "::ffff:" to ipv4 addresses!

# IPs that aren't allowed to generate *any* wiki pages
global $access_blacklist_ips;
$access_blacklist_ips = array(
	     '::ffff:202.38.188.69', # someone wget -r 'ing the entire site 21 March 2005
	     '::ffff:202.38.188.68', # ditto, Apr 6
	     '2001:250:ffff:ffff:204:76ff:fe98:5e34', # ditto (same institution?)
	     '::ffff:24.255.112.246', # stuck in a loop requesting /AddingPages?action=LikePages
	     '::ffff:84.115.68.187',  # ditto, 24 Mar 2005
	     '::ffff:211.54.97.59',   # ditto, Mar 27
	     '::ffff:217.147.35.19',
);

# accesses based on User-Agent that aren't allowed to generate *any* pages
global $access_blacklist_agents;
$access_blacklist_agents = array (
             'DTS Agent',
# South African crawler (from 168.209.98.35 above) has " in user agent
# apache turns " into \"
# excessive crawling on 21, 25, 27th April 2004
           '\"Mozilla/4.0 (Compatible, MSIE 6.0, ANC 2004, Super Windows 98)\"',# getting lots of diffs/backlinks, no referrer... weird. 22 June 2004
             'Mozilla/4.0 (compatible; MSIE 4.0; Windows NT; ....../1.0 )',
	     'WebCopier v4.2', # crawling whole site (inc old vers) 11 Apr 2005
	     'aipbot/1.0 (aipbot; http://www.aipbot.com; aipbot@aipbot.com)', # seems to be a dodgy referer-type site
		# used by some distributed referrer spamming software
	     'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; KITV4.7 Wanadoo)',
);



# IPs that aren't allowed to edit pages, due to past abuse.
# (this is only checked when they try to save, not on edit).
# this is a regexp, so you can block a range
global $edit_blacklist_ip_rxs;
$edit_blacklist_ip_rxs = array(
		# chinese spammer uses the name ZhixiongKang
		# the whole range is 60.24/16 - 60.30/16, but spammer
		# seems to only come from this part of it:
		'^::ffff:60\.26\.',
		#'^::ffff:60\.2[456789]\.',	#netname:      CNCGROUP-TJ
		#'^::ffff:60\.30\.',		#netname:      CNCGROUP-TJ
		'^::ffff:61\.14[89]',		#netname:      CNCGROUP-BJ
		# and this range....
		'^::ffff:221\.19[678]',		#netname:      CNCGROUP-TJ
		# and 218.69.195.{81,46}
		'^::ffff:218\.69\.195\.',	#netname: CNCGROUP-TJ
		# also 222.47.27.161 # China railway 222.32.0.0/11
		# Chinese spammer (signed as "LiYan") 18 May 2005
		'^::ffff:221\.196\.99\.50', # netname: CHINANET-SH (222.64/11?)
		'^::ffff:222\.65\.189',
		# netname: CHINANET-SH 218.78-83
		'^::ffff:218\.80\.66\.188',
		'^::ffff:218\.82\.13',
		# a different, incompetent chinese spammer 20 May 2005
		'^::ffff:211\.90\.107\.110',    # UNICOM
		
		'::ffff:213\.228\.74\.59', # drug spammer, 6 Apr 2005
		'::ffff:65\.118\.159\.176', # QWEST customer spammer 10 Apr
	       	);



# is this person allowed to access *any* wiki pages?
function check_access_blacklist() {
	global $access_blacklist_agents;
	global $access_blacklist_ips;
    if ( (isset($_SERVER["HTTP_USER_AGENT"]) &&
          in_array($_SERVER["HTTP_USER_AGENT"], $access_blacklist_agents))
         ||
         (isset($_SERVER["REMOTE_ADDR"]) &&
          in_array($_SERVER["REMOTE_ADDR"], $access_blacklist_ips))
       ) {
        Header("HTTP/1.0 403 Suspected email address harvester");
        Header("Pragma: nocache");
?>
<html>
 <head>
  <title>Forbidden: Suspected email address harvester</title>
 </head>
 <body>
 <h1>Forbidden</h1>
 <p>You have been forbidden access to this site, as it is believed you are a
   spam address-harvester robot that doesn't respect/request robots.txt or
   ratelimit your queries to a reasonable level.</p>
 <p>Alternatively, your address may have been banned for attempting to use
   well-known exploits against our server, or attempting to add
   links of a commercial nature to our editable pages.</p>
 <p>If you believe this is in error, please contact us through one of the
 following methods:
 <ul>
  <li> Email the web master at this site</li>
  <li> Subscribe and post asking for help from the wlug mailing list 
   (http://list.waikato.ac.nz)</li>
  <li> Ask on <a href="irc://irc.undernet.org/wlug">#wlug</a> on the Undernet IRC network</li>
 </ul>
 </p>
 <p> Sorry for any inconvenience this may have caused,<br>
 The WLUG Website maintainer</p>
 <hr>
 Since the spammers want email addresses, heres one for them (randomly
 generated so they never go anywhere):
 <a href="mailto:<?
   $common_chars=array('a','e','r','s','t','n','i', 'h');
   $num_chars=sizeof($common_chars);

   $address="";
   $length=rand(3,8);
   for($i=0;$i<$length;$i++) { $address .= chr(rand(97,122)); }
   $address .= "@";
   $length=rand(6,16);
   for($i=0;$i< $length;$i++) {$address .= $common_chars[rand(0, $num_chars)]; }
   if (rand(0,1)) {$address .= ".com";} else {$address .= ".net";}
   print "$address\">$address</a>\n";
 ?>
 </body>
</html>
<?
        exit; 
    }
}




# doesn't return if spam detected
function check_edit_spam( $current_text, $new_text ) {
	global $edit_blacklist_ip_rxs;

	$is_spam = 0;
	$is_chinese_spam = 0;


	$ip_addr = $_SERVER['REMOTE_ADDR'];
	foreach ( $edit_blacklist_ip_rxs as $banned_range ) {
		if ( ereg($banned_range, $ip_addr) ) {
			$is_spam = 1;
			$is_chinese_spam = 1;
			break;
		}
	}


	if ( $is_spam == 0 ) {
		# check the number of new http links added by this edit
		# mostly copied from phpwiki cvs source
		$current_link_count = substr_count( $current_text, "http://" );
		$new_link_count = substr_count( $new_text, "http://" );
		if ( $new_link_count - $current_link_count >= 20 ) {
			# too many links added in this edit
			$is_spam = 1;
		}
	}

	if ($is_spam == 0) {
		/* get chinese spammers to fuck off - look for chinese chars
		 * - match 3byte utf-8 chars starting with 0xe7 or
		 * 0xe9 as well as html &#.....; entities */

		$chinese_char = '[\xE6\xE7\xE9][\x80-\xbf]{2}|&#[23]\d{4};';

		# look inside [...] and http://.* for chars...
		if (preg_match("!(?:http://|\[)[^\]]+(?:$chinese_char)!",
					$new_text)) {
		       	$is_spam = 1;
			$is_chinese_spam = 1;
	       	}
	        # look just before hrefs
		elseif (preg_match("!$chinese_char\s*\[?http://!", $new_text)) {
			$is_spam = 1;
			$is_chinese_spam = 1;
		}
	}


	if ($is_spam == 0) {
		# doesn't appear to be comment spam
		return false;
	}

# try to dissuade the chinese wiki spammer(s) from visiting us. May 2005
	if (0 && $is_chinese_spam == 1) {
		header('HTTP/1.0 301 Moved');
# redirect to 'join falun gong' website
		header('Location: http://www.fofg.org/chinese/index.php');
#		header('Location: http://lemonparty.org/');
		exit(0);
	}
	
	# give a non-200 return code, so it's easier to search
	# for in apache's access log :)
	header('HTTP/1.0 201 Created');
	header("Content-Type: text/html; charset=" . CHARSET);

	# let them waste a few seconds....
	sleep(20);

	global $Theme;
	require_once('lib/PageType.php');
	require_once('lib/HtmlElement.php');
	require_once('lib/stdlib.php');

	$body = HTML();

	$body->pushContent('Your edit was not saved due to our anti-spam filters. Possible reasons include: ');
	$list=HTML::ul();
	$list->pushContent(
			HTML::li('Past abuse from your IP address/sub-net'),
			HTML::li('Too many suspicious links in the content'),
			HTML::li('Otherwise suspicious content.')
			);
	$body->pushContent( $list );
#stdlib
	$par= HTML::p();
	$par->pushContent('If you think we\'ve made an error, please contact us: ', HTML::br());
	$par->pushContent( LinkURL('irc://irc.undernet.org/wlug'),
		       	', email:',
			LinkURL('mailto:admin@wlug.org.nz'),
		       	', or jabber:',
			LinkURL('jid://kinko@jabber.meta.net.nz'), '. Thank you.');
	$body->pushContent( $par );


	$template = Template('top',
			array('CONTENT' => $body,
				'TITLE' => 'Error',
				'HEADER' => 'Error Saving Revision')
			);
	$body = Template('body');
	$template->printExpansion($body);
	# return true; # ??
	exit;
}

?>
