<?php
// display.php: fetch page or get default content
rcs_id('$Id: display.php,v 1.30 2002/03/06 03:52:22 carstenklapp Exp $');

require_once('lib/Template.php');
require_once('lib/BlockParser.php');

/**
 * Guess a short description of the page.
 *
 * Algorithm:
 *
 * This algorithm was suggested on MeatballWiki by
 * Alex Schroeder <kensanata@yahoo.com>.
 *
 * Use the first paragraph in the page which contains at least two
 * sentences.
 *
 * @see http://www.usemod.com/cgi-bin/mb.pl?MeatballWikiSuggestions
 */
function GleanDescription ($rev) {
    $two_sentences
        = pcre_fix_posix_classes("/[.?!]\s+[[:upper:])]"
                                 . ".*"
                                 . "[.?!]\s*([[:upper:])]|$)/sx");

    $content = $rev->getPackedContent();

    // Iterate through paragraphs.
    while (preg_match('/(?: ^ \w .* $ \n? )+/mx', $content, $m)) {
        $paragraph = $m[0];

        // Return paragraph if it contains at least two sentences.
        if (preg_match($two_sentences, $paragraph)) {

	    # need to escape this for html, eg " in meta="..."
	    # should also check for < and >
	    $paragraph=preg_replace('/["<\[>\]]/', '', $paragraph);

            return preg_replace("/\s*\n\s*/", " ", trim($paragraph));
        }

        $content = substr(strstr($content, $paragraph), strlen($paragraph));
    }
    return '';
}


function actionPage(&$request, $action) {
    global $Theme;

    $pagename = $request->getArg('pagename');
    $version = $request->getArg('version');

    $page = $request->getPage();
    $revision = $page->getCurrentRevision();

    $dbi = $request->getDbh();
    $actionpage = $dbi->getPage($action);
    $actionrev = $actionpage->getCurrentRevision();

    $pagetitle = HTML(fmt("%s: %s", $actionpage->getName(),
                          $Theme->linkExistingWikiWord($pagename, false, $version)));

    header("Content-Type: text/html; charset=" . CHARSET);
    # don't override if it has already been set
    $mtime=$request->getArg('last_modified_timestamp');
    if (! $mtime) {
	$mtime=$revision->get('mtime');
	    header('Expires: Sat 19 Mar 2005 11:40:04 GMT');
    } else {
	    header('Expires: Thu 17 Mar 2005 11:40:04 GMT');
    }
    header("Last-Modified: ".Rfc2822DateTime($mtime));
    header("Cache-Control: must-revalidate");

    require_once('lib/PageType.php');
    $transformedContent = PageType($actionrev);
    $template = Template('browse', array('CONTENT' => $transformedContent));

    // added by John McPherson so google etc don't waste time with
    // diffs/backlinks/info/pagehistory etc
    $args=array('ROBOTS_CONTENT' => "noindex,nofollow");
    GeneratePage($template, $pagetitle, $revision, $args);
    flush();
}

function displayPage(&$request, $tmpl = 'browse') {
    global $HTTP_IF_MODIFIED_SINCE;
    global $HTTP_CACHE_CONTROL;
    global $HTTP_PRAGMA;
    $args=array(); // passed to GeneratePage (John McPherson)

    $pagename = $request->getArg('pagename');

    // John McPherson - new template for sidebar...
    if ($pagename == _("SideBarFrame")) {
	$tmpl='sidebar';
	$template = Template('sidebar');
	printXML($template);
	exit;
    }

    
    $version = $request->getArg('version');
    $page = $request->getPage();
    if ($version) {
        $args['ROBOTS_CONTENT']="noindex,nofollow"; // Tell google not to index old versions... (jrm)
	$revision = $page->getRevision($version);
        if (!$revision)
            NoSuchRevision($request, $page, $version);
    }
    else {
        $revision = $page->getCurrentRevision();
    }

    $splitname = split_pagename($pagename);
    $pagetitle = HTML::a(array('href' => WikiURL($pagename,
                                                 array('action' => _("BackLinks"))),
			       'rel' => 'nofollow', # jrm - wlug wiki
                               'class' => 'backlinks'),
                         $splitname);
    $pagetitle->addTooltip(sprintf(_("BackLinks for %s"), $pagename));

    require_once('lib/PageType.php');
    $transformedContent = PageType($revision);
    $template = Template($tmpl, array('CONTENT' => $transformedContent));

    #### customised for wlug wiki - 304 not modified if I-M-S header found
    header("Content-Type: text/html; charset=" . CHARSET);
    header("Cache-Control: public must-revalidate");

    # should we disable 304 Not Modified caching responses?
    $nocache = 0;
	
    // First, do checks based on pagename
    if ( $pagename == _("RecentChanges") || $pagename == _("RecentEdits")) {
	$args['ROBOTS_CONTENT']="noindex,follow";
	$nocache = 1; # don't cache these pages
    } else if ($pagename == _("RecentVisitors") || $pagename == _("SandBox")) {
	     $args['ROBOTS_CONTENT']="noindex,nofollow";
    } else if (ereg('^Category', $pagename)) {
	# don't cache Category* pages since they are calculated by BackLinks
	$nocache = 1;
    }

    if ($nocache == 0) {
	if(isset($HTTP_CACHE_CONTROL)) {
	    if (preg_match ('/no-cache/i', $HTTP_CACHE_CONTROL) ) {
		$nocache = 1;
	    }
	} else if (isset($HTTP_PRAGMA)) {
	    if (preg_match ('/no-cache/i', $HTTP_PRAGMA) ) {
		$nocache = 1;
	    }
	}
    }


    # don't use page db saved time if a plugin or something has done
    # something smarter and set this request argument. (jrm wlug wiki)
    $mtime=$request->getArg('last_modified_timestamp');
    if (! $mtime) {
	$mtime=$revision->get('mtime');
    }

    if (! $nocache && $HTTP_IF_MODIFIED_SINCE) {
	// this request has an If-Modified-Since header
	// sometimes this has "; length=xxxx" appended... not sure why
	// but get rid of it for parsing
	$laststrdate=preg_replace('|;.*$|', '', $HTTP_IF_MODIFIED_SINCE);
	// strtotime should handle timezones, and appears to convert to utc
	$lastmtime=strtotime($laststrdate); // returns -1 if can't parse
	if ($lastmtime >= $mtime) {
	    header("HTTP/1.1 304 Slackers haven't updated this page yet");
	    exit;
	}
    }

    # don't print this header if we don't want the page cached, in case
    # any proxies try to be smart
    if (! $nocache) {
	header("Last-Modified: ".Rfc2822DateTime($mtime));
    }

    $args['PAGE_DESCRIPTION']=GleanDescription($revision);
    GeneratePage($template, $pagetitle, $revision, $args);
    flush();

    $page->increaseHitCount();
}

// For emacs users
// Local Variables:
// mode: php
// tab-width: 8
// c-basic-offset: 4
// c-hanging-comment-ender-p: nil
// indent-tabs-mode: nil
// End:
?>
