<?php // -*-php-*-
/**
 * based on the HelloWorld plugin
 * written by PerryLorier, ldap support added by JohnMcPherson
 * Usage:
 * <?plugin WlugMember ?>
 */

// Constants are defined before the class.
if (!defined('THE_END'))
    define('THE_END', "!");

class WikiPlugin_WlugMember
extends WikiPlugin
{
    // Four required functions in a WikiPlugin.

    function getName () {
        return _("WlugMember");
    }

    function getDescription () {
        return _("Displays a logo if this is a current WlugMember");

    }
    // Establish default values for each of this plugin's arguments.
    function getDefaultArguments() {
        return array('name'	  => "[pagename]");
    }

    function error($msg) {
	$div = HTML::div(array('class' => 'errors'));
	$err = "WlugMember ldap error: ";
	$div->pushContent($err . $msg);
	return $div;
    }

    function run($dbi, $argstr, $request) {
        extract($this->getArgs($argstr, $request));

        // Any text that is returned will not be further transformed,
        // so use html where necessary.

	if(!function_exists('ldap_connect')) {
		return $this->error("missing php ldap?");
	}
	$host = 'localhost';

	$conn = ldap_connect($host);
	if (! $conn) {
		return $this->error("connecting to ldap server");
	}
	$bind = ldap_bind($conn);
	if (! $bind) {
		return $this->error("binding to ldap server");
	}

	
	/* Ideally we would work out the common name from the wiki word, but
	   that isn't always possible (eg surnames with Mc or van/der prefixes,
	   and we can't always get the capitalisation right. so we'll have to
	   go through all the names and strncasecmp them... sigh
	 */
	$base = 'ou=People,dc=wlug,dc=org,dc=nz';
	$filter = 'cn=*';
	$attrs = array('cn', 'description');

	$result = @ldap_search( $conn, $base, $filter, $attrs,
			0, # attributes only (?)
			0, # size limit/max returned
			0, # time limit
			LDAP_DEREF_NEVER);

	if(! $result) {
		return $this->error(ldap_error($conn));
	}
	$entries = @ldap_get_entries( $conn, $result);

        $html = HTML::img(); 
	$html->setAttr('src',"/phpwiki/themes/WLUG/images/favicon.ico");
	$found = 0;

	foreach ($entries as $entry) {
		$thisname = $entry['cn'][0];

# get name in the same format that wiki uses
		$thisname = str_replace( array(" ", "-"),
				"",
				$thisname);
# matches the current page name?
		if ( strcasecmp($thisname, $name)==0) {
			$html->setAttr('style',"border: 4px outset gold; float: right;");
			$desc = $entry['description'][0];
			$descfields = explode(",", $desc);
			$paid = $descfields[1];
			if ( count($descfields)==4 &&
			      strcasecmp($descfields[3], 'expired') == 0) {
                                $html->setAttr('title',"$name is a past member of the Waikato Linux Users Group");
			} elseif(strcasecmp($paid, "Yes") == 0)
                                $html->setAttr('title',"$name is a paid member of the Waikato Linux Users Group");
                        else
                                $html->setAttr('title',"$name is a member of the Waikato Linux Users Group (pending payment)");
			$found = 1;
			break;
		}
	}

	ldap_unbind($conn);

	if ($found == 0) {
		$html->setAttr('title', "$name is currently not a paid member of the Waikato Linux Users Group");
		$html->setAttr('style',"border: 1px solid black; float: right;");
	}
        return $html;
    }
};

// For emacs users
// Local Variables:
// mode: php
// tab-width: 8
// c-basic-offset: 4
// c-hanging-comment-ender-p: nil
// indent-tabs-mode: nil
// End:
?>
